<?php
namespace Teknisa\TCO\Controllers;

use Zeedhi\Framework\Controller\Report as ReportController;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;

class Tco01200CardapioDaPropostaReport extends ReportController {
    protected $reportService;
    protected $birtStrategy;

    protected $reportMapping = array(
        'TCO01200' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRORG'               => 'P_NRORG',
                'NRCARDAPIOTRAB'      => 'P_NRCARDAPIOTRAB',
                'CDARVPRODINICIAL'    => 'P_CDARVPRODINICIAL',
                'CDARVPRODFINAL'      => 'P_CDARVPRODFINAL',
                'CDPRATOINICIAL'      => 'P_CDPRATOINICIAL',
                'CDPRATOFINAL'        => 'P_CDPRATOFINAL'
            )
        ),
        'TCO01201' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRORG'             => 'P_NRORG',
                'NRCARDAPIOTRAB'    => 'P_NRCARDAPIOTRAB',
                'CDARVPRODINICIAL'  => 'P_CDARVPRODINICIAL',
                'CDARVPRODFINAL'    => 'P_CDARVPRODFINAL',
                'CDPRATOINICIAL'    => 'P_CDPRATOINICIAL',
                'CDPRATOFINAL'      => 'P_CDPRATOFINAL'
            )
        ),
        'TCO01202' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRCARDAPIOTRAB'      => 'P_NRCARDAPIOTRAB',
                'NRORG'               => 'P_NRORG'
            )
        ),
        'TCO01203' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRCARDAPIOTRAB'      => 'P_NRCARDAPIOTRAB',
                'NRORG'               => 'P_NRORG'
            )
        ),
        'TCO01204' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRCARDAPIOTRAB'      => 'P_NRCARDAPIOTRAB',
                'NRORG'               => 'P_NRORG'
            )
        ),
        'TCO01205' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRORG'                 => 'P_NRORG',
                'NRCARDAPIOTRAB'        => 'P_NRCARDAPIOTRAB',
                'COLUNAS'               => 'P_COLUNAS',
                'NRSEMANA_INICIAL'      => 'P_NRSEMANA_INICIAL',
                'NRDIASEMANA_INICIAL'   => 'P_NRDIASEMANA_INICIAL',
                'NRSEMANA_FINAL'        => 'P_NRSEMANA_FINAL',
                'NRDIASEMANA_FINAL'     => 'P_NRDIASEMANA_FINAL',
                'CDARVPRODINICIAL'      => 'P_CDARVPRODINICIAL',
                'CDARVPRODFINAL'        => 'P_CDARVPRODFINAL',
                'CDPRATOINICIAL'        => 'P_CDPRATOINICIAL',
                'CDPRATOFINAL'          => 'P_CDPRATOFINAL'
            )
        ),
        'TCO01206' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRORG'               => 'P_NRORG',
                'NRCARDAPIOTRAB'      => 'P_NRCARDAPIOTRAB',
                'COLUNAS'             => 'P_COLUNAS',
                'NRSEMANA_INICIAL'    => 'P_NRSEMANA_INICIAL',
                'NRDIASEMANA_INICIAL' => 'P_NRDIASEMANA_INICIAL',
                'NRSEMANA_FINAL'      => 'P_NRSEMANA_FINAL',
                'NRDIASEMANA_FINAL'   => 'P_NRDIASEMANA_FINAL',
                'CDARVPRODINICIAL'    => 'P_CDARVPRODINICIAL',
                'CDARVPRODFINAL'      => 'P_CDARVPRODFINAL',
                'CDPRATOINICIAL'      => 'P_CDPRATOINICIAL',
                'CDPRATOFINAL'        => 'P_CDPRATOFINAL'
            )
        ),
        'TCO01207' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRORG'                 => 'P_NRORG',
                'NRCARDAPIOTRAB'        => 'P_NRCARDAPIOTRAB',
                'COLUNAS'               => 'P_COLUNAS',
                'NRSEMANA_INICIAL'      => 'P_NRSEMANA_INICIAL',
                'NRDIASEMANA_INICIAL'   => 'P_NRDIASEMANA_INICIAL',
                'NRSEMANA_FINAL'        => 'P_NRSEMANA_FINAL',
                'NRDIASEMANA_FINAL'     => 'P_NRDIASEMANA_FINAL',
                'CDARVPRODINICIAL'      => 'P_CDARVPRODINICIAL',
                'CDARVPRODFINAL'        => 'P_CDARVPRODFINAL',
                'CDPRATOINICIAL'        => 'P_CDPRATOINICIAL',
                'CDPRATOFINAL'          => 'P_CDPRATOFINAL'
            )
        ),
        'TCO01208' => array(
            'strategy' => 'BIRT',
            'parameterMapping' => array(
                'NRCARDAPIOTRAB'      => 'P_NRCARDAPIOTRAB',
                'NRORG'               => 'P_NRORG'
            )
        )
    );

    public function __construct($reportService, $birtStrategy, $environment){
        $this->reportService = $reportService;
        $this->birtStrategy = $birtStrategy;
        $this->environment = $environment;
    }

    public function beforeProcessReport(Row $request, Response $response) {
        $row = $request->getRow();
       
        $row['COLUNAS'] = empty($row['NMTIPOPRAT']) ? "T" : implode('_', $row['NMTIPOPRAT']);
        
        $row['NRSEMANA_INICIAL'] = isset($row['NRSEMANA_INICIAL']) ? $row['NRSEMANA_INICIAL'] : null;
        $row['NRDIASEMANA_INICIAL'] = isset($row['NRDIASEMANA_INICIAL']) ? $row['NRDIASEMANA_INICIAL'] : null;
        $row['NRSEMANA_FINAL'] = isset($row['NRSEMANA_FINAL']) ? $row['NRSEMANA_FINAL'] : null;
        $row['NRDIASEMANA_FINAL'] = isset($row['NRDIASEMANA_FINAL']) ? $row['NRDIASEMANA_FINAL'] : null;
        
        $row['NRORG'] = $this->environment->getNrOrg();
        
        $row['CDPRATOINICIAL'] = $row['CDPRATOINICIAL'] ?? "0"; // check key exists
        $row['CDPRATOINICIAL'] = empty($row['CDPRATOINICIAL']) ? "0" : $row['CDPRATOINICIAL']; 
            // check empty value
        
        $row['CDARVPRODINICIAL'] = $row['CDARVPRODINICIAL'] ?? "0";
        $row['CDARVPRODINICIAL'] = empty($row['CDARVPRODINICIAL']) ? "0" : $row['CDARVPRODINICIAL'];

        $row['CDPRATOFINAL'] = $row['CDPRATOFINAL'] ?? "";

        $row['CDARVPRODFINAL'] = $row['CDARVPRODFINAL'] ?? "";
        
        $row['CDPRATOFINAL'] = str_pad($row['CDPRATOFINAL'], 10, "z");
        $row['CDARVPRODFINAL'] = str_pad($row['CDARVPRODFINAL'], 20, "z");

        $row['__report_name'] = $row['REPORT'];
         
        $this->birtStrategy->setReportFormat($row['BIRTTYPE']);
    }
}