<?php

namespace Teknisa\TCO\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Tco01001ElaboracaoCardapioService extends \Teknisa\GER\Controllers\TeknisaCrud{
    
    protected $crudService;
    protected $environment;
    protected $entityManager;

    public function __construct($crudService, $parameterBag, $environment, $entityManager) {
        $this->crudService = $crudService;
        $this->environment = $environment;
        $this->entityManager = $entityManager;
    }
    
    public function retornaCardapioBaseExistente($row) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT 'TCO01000_CARDAPIO_BASE_EXISTENTE' AS MESSAGECODE,
                       C.NRCARDAPIOBASE,
                       C.CDCARDAPIOBASE,
                       TO_CHAR(C.DTINIVIGENCIA, 'DD/MM/YYYY') AS DTINIVIGENCIA,
                       TO_CHAR(C.DTFIMVIGENCIA, 'DD/MM/YYYY') AS DTFIMVIGENCIA
                  FROM CARDAPIOBASE C
                 WHERE (C.NRSERVICOPROPOSTA = :NRSERVICOPROPOSTA)
                   AND C.NRCARDAPIOBASE <> :NRCARDAPIOBASE
                   AND EXISTS ( SELECT 1 
                                  FROM SEGMERCARDBASE
                                 WHERE NRCARDAPIOBASE = C.NRCARDAPIOBASE
                                   AND CDSEGMER IN (SELECT CDSEGMER
                                                      FROM SEGMERCARDBASE 
                                                     WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE))
                   AND (C.NRPADRAOQUALIDADE = :NRPADRAOQUALIDADE)
                   AND (C.IDATIVO = 'S')
                   AND ((C.DTINIVIGENCIA BETWEEN :DTINIVIGENCIA AND :DTFIMVIGENCIA)
                         OR (C.DTFIMVIGENCIA BETWEEN :DTINIVIGENCIA AND :DTFIMVIGENCIA)
                         OR (:DTINIVIGENCIA BETWEEN C.DTINIVIGENCIA AND C.DTFIMVIGENCIA)
                         OR (:DTFIMVIGENCIA BETWEEN C.DTINIVIGENCIA AND C.DTFIMVIGENCIA))
                   AND C.NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE"     => isset($row["NRCARDAPIOBASE"]) ? $row["NRCARDAPIOBASE"]       : null,
            "NRSERVICOPROPOSTA"  => isset($row["NRSERVICOPROPOSTA"]) ? $row["NRSERVICOPROPOSTA"] : null,
            "NRPADRAOQUALIDADE"  => isset($row["NRPADRAOQUALIDADE"]) ? $row["NRPADRAOQUALIDADE"] : null,
            "DTINIVIGENCIA"      => isset($row["DTINIVIGENCIA"])     ? $row["DTINIVIGENCIA"]     : null,
            "DTFIMVIGENCIA"      => isset($row["DTFIMVIGENCIA"])     ? $row["DTFIMVIGENCIA"]     : null,
            "NRORG"              => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        
        return $result;
    }
    
    public function procedureImportarCardBase($row) {
        $connection = $this->entityManager->getConnection();
        $procedure = new StoredProcedure($connection, 'IMPORTAR_CARDBASE');
        
        $procedure->addParam(new Param('P_NRCARDAPIOBASEORI',  Param::PARAM_INPUT, $row['NRCARDAPIOBASE'],              Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRCARDAPIOBASEDEST', Param::PARAM_INPUT, $row['NRCARDAPIOBASEDEST'],          Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_COLUNAS',            Param::PARAM_INPUT, $row['NRCONFIGCARDBASE'],            Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_COMPOSICAO',         Param::PARAM_INPUT, $row['COPIAR_PRATOS'],               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_REGRAS',             Param::PARAM_INPUT, $row['COPIAR_REGRAS'],               Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',         Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG',              Param::PARAM_INPUT, $this->environment->getNrOrgTrab(),  Param::PARAM_TYPE_INT));
        $result = $procedure->execute();
        
        
        
    }
    
    public function procedureRemovePratoCardBase($row) {
        $connection = $this->entityManager->getConnection();
        
        $procedure = new StoredProcedure($connection, 'DELETA_PRATOS_CARDBASE');
        
        $procedure->addParam(new Param('P_NRCARDAPIOBASE', Param::PARAM_INPUT, $row['NRCARDAPIOBASE'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DIAS',         Param::PARAM_INPUT, $row['DSSEMANA'],   Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CONFIGURACAO', Param::PARAM_INPUT, $row['NMTIPOPRAT'], Param::PARAM_TYPE_STR));
        
        $result = $procedure->execute();
    }
    
    public function procedureCopiaPratos($row){
        $connection = $this->entityManager->getConnection();
        
        $procedure = new StoredProcedure($connection, 'COPIA_PRATO_CARDBASE');
        
        $procedure->addParam(new Param('P_NRCARDAPIOBASE',         Param::PARAM_INPUT, $row['NRCARDAPIOBASE'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCOMPCARDBASE',         Param::PARAM_INPUT, $row['NRCOMPCARDBASE'],      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRDIACARDBASEDEST',      Param::PARAM_INPUT, $row['NRDIACARDBASE'],       Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRCONFIGCARDBASEDEST',   Param::PARAM_INPUT, $row['NRCONFIGCARDBASE'],    Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRORG',                  Param::PARAM_INPUT, $this->environment->getNrOrgTrab(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR',             Param::PARAM_INPUT, $this->environment->getCdOperador(), Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SUBTITUIR',              Param::PARAM_INPUT, 'S',                         Param::PARAM_TYPE_STR));
        
        $result = $procedure->execute();
    }
    
    public function procedureReplicaPratos($row){
        $connection = $this->entityManager->getConnection();
        
        $procedure = new StoredProcedure($connection, 'REPLICA_PRATO_CARDBASE');
        $procedure->addParam(new Param('P_NRCARDAPIOBASE', Param::PARAM_INPUT, $row['NRCARDAPIOBASE'],             Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRCOMPCARDBASE', Param::PARAM_INPUT, $row['NRCOMPCARDBASE'],             Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_NRORG',          Param::PARAM_INPUT, $this->environment->getNrOrgTrab(), Param::PARAM_TYPE_INT));
        $procedure->addParam(new Param('P_CDOPERADOR',     Param::PARAM_INPUT, $this->environment->getCdOperador(),Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_DOM',            Param::PARAM_INPUT, $row['P_DOM'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEG',            Param::PARAM_INPUT, $row['P_SEG'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_TER',            Param::PARAM_INPUT, $row['P_TER'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QUA',            Param::PARAM_INPUT, $row['P_QUA'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_QUI',            Param::PARAM_INPUT, $row['P_QUI'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SEX',            Param::PARAM_INPUT, $row['P_SEX'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SAB',            Param::PARAM_INPUT, $row['P_SAB'],                      Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_SUBTITUIR',      Param::PARAM_INPUT, $row['SOBREPOR'],                   Param::PARAM_TYPE_STR));
        
        $result = $procedure->execute();
    }
    
    public function diasAExcluir($nrcardapiobase, $dias) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRDIACARDBASE,
                       NRCARDAPIOBASE,
                       NRSEMANA,
                       NRDIASEMANA,
                       NRORG,
                       IDATIVO
                  FROM DIACARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                   AND NRORG = :NRORG
                 ORDER BY NRSEMANA DESC,
                       NRDIASEMANA DESC";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE"     => $nrcardapiobase,
            "NRORG"              => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        $diasAExcluir = array();
        
        for($i = 0; $i < $dias; ++$i) {
            array_push($diasAExcluir, $result[$i]);
        }
        
        return $diasAExcluir;
        
    }
    
    public function findComposicaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRCOMPCARDBASE,
                       NRCARDAPIOBASE,
                       NRDIACARDBASE,
                       NRCONFIGCARDBASE,
                       NRORG
                  FROM COMPCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                   AND INSTR(:NRDIACARDBASE, NRDIACARDBASE) > 0
                   AND NRORG = :NRORG";
                   
        $query = $connection->prepare($sql);
        
        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    public function findPratosComposicaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT COMPCARDBASE.NRDIACARDBASE,
                       PRATCOMPCARDBASE.NRPRATCOMPCARDBASE,
                       PRATCOMPCARDBASE.NRCOMPCARDBASE,
                       PRATCOMPCARDBASE.CDFILAUXPRAT,
                       PRATCOMPCARDBASE.CDPRATO,
                       PRATCOMPCARDBASE.NRORG
                  FROM PRATCOMPCARDBASE
                 INNER JOIN COMPCARDBASE
                    ON COMPCARDBASE.NRCOMPCARDBASE = PRATCOMPCARDBASE.NRCOMPCARDBASE 
                 WHERE COMPCARDBASE.NRCARDAPIOBASE = :NRCARDAPIOBASE 
                   AND INSTR(:NRDIACARDBASE, COMPCARDBASE.NRDIACARDBASE) > 0
                   AND PRATCOMPCARDBASE.NRORG = :NRORG";
                   
        $query = $connection->prepare($sql);
        
        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );
        
        $query->execute($params);
        
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);
        return $result;
    }
    
    public function findRegioesPratoComposicaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRREGIAOPRATCARDBASE,
                       NRPRATCOMPCARDBASE,
                       CDREGIAO,
                       NRORG
                  FROM REGIAOPRATCARDBASE
                 WHERE NRPRATCOMPCARDBASE IN (SELECT PRATCOMPCARDBASE.NRPRATCOMPCARDBASE
                                                FROM PRATCOMPCARDBASE
                                               INNER JOIN COMPCARDBASE
                                                  ON COMPCARDBASE.NRCOMPCARDBASE = PRATCOMPCARDBASE.NRCOMPCARDBASE
                                               INNER JOIN DIACARDBASE
                                                  ON DIACARDBASE.NRDIACARDBASE = COMPCARDBASE.NRDIACARDBASE
                                               WHERE COMPCARDBASE.NRCARDAPIOBASE = :NRCARDAPIOBASE 
                                                 AND INSTR(:NRDIACARDBASE, DIACARDBASE.NRDIACARDBASE) > 0
                                                 AND PRATCOMPCARDBASE.NRORG = :NRORG)";

        $query = $connection->prepare($sql);

        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);

        return $result;
    }
    
    public function findItemAdesaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRITEMADESAOCBASE,
                       NRADESAOCARDBASE,
                       NRCONFIGCARDBASE,
                       NRPORCIONAMENTO,
                       NRORG
                  FROM ITEMADESAOCBASE
                 WHERE NRADESAOCARDBASE IN ( SELECT NRADESAOCARDBASE
                                               FROM ADESAOCARDBASE
                                              WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                                                AND INSTR(:NRDIACARDBASE, NRDIACARDBASE) > 0 )
                   AND NRORG = :NRORG";

        $query = $connection->prepare($sql);

        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);

        return $result;
    }
    
    public function findAtividadeClienteAdesaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRATIVCLIADESAOCBASE,
                       NRADESAOCARDBASE,
                       NRATIVCLISEGMER,
                       NRORG
                  FROM ATIVCLIADESAOCBASE
                 WHERE NRADESAOCARDBASE IN ( SELECT NRADESAOCARDBASE
                                               FROM ADESAOCARDBASE
                                              WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                                                AND INSTR(:NRDIACARDBASE, NRDIACARDBASE) > 0 )
                   AND NRORG = :NRORG";

        $query = $connection->prepare($sql);

        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);

        return $result;
    }
    
    public function findSegmentoMercadoAdesaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRSEGMERADESAOCBASE,
                       NRADESAOCARDBASE,
                       CDSEGMER,
                       NRORG
                  FROM SEGMERADESAOCBASE
                 WHERE NRADESAOCARDBASE IN ( SELECT NRADESAOCARDBASE
                                               FROM ADESAOCARDBASE
                                              WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                                                AND INSTR(:NRDIACARDBASE, NRDIACARDBASE) > 0 )
                   AND NRORG = :NRORG";

        $query = $connection->prepare($sql);

        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);

        return $result;
    }
    
    public function findAdesaoCardapioBase($rows) {
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT NRADESAOCARDBASE,
                       NRCARDAPIOBASE,
                       CDADESAOCARDBASE,
                       NMADESAOCARDBASE,
                       CDGRTIPOPR,
                       NRDIACARDBASE,
                       NRORG
                  FROM ADESAOCARDBASE
                 WHERE NRCARDAPIOBASE = :NRCARDAPIOBASE
                   AND INSTR(:NRDIACARDBASE, NRDIACARDBASE) > 0
                   AND NRORG = :NRORG";

        $query = $connection->prepare($sql);

        $diasAExcluirParam = "";

        foreach($rows as $row) {
            $diasAExcluirParam .= "_".$row["NRDIACARDBASE"];
        }

        $params = array(
            "NRCARDAPIOBASE" => $rows[0]["NRCARDAPIOBASE"],
            "NRDIACARDBASE"  => $diasAExcluirParam,
            "NRORG"          => $this->environment->getNrOrgTrab()
        );

        $query->execute($params);
        $result = $query->fetchAll(\PDO::FETCH_ASSOC);

        return $result;
    }
    
    public function getUltimaSemana($nrcardapiobase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT MAX(NRSEMANA) AS NRSEMANA
                  FROM DIACARDBASE
                 WHERE DIACARDBASE.NRCARDAPIOBASE = :NRCARDAPIOBASE
                   AND DIACARDBASE.NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE" => $nrcardapiobase,
            "NRORG"          => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result["NRSEMANA"];
    }
    
    public function getDiasExistentes($nrcardapiobase){
        $connection = $this->entityManager->getConnection();
        $sql = "SELECT COUNT(1) AS DIASEXISTENTES
                  FROM DIACARDBASE
                 WHERE DIACARDBASE.NRCARDAPIOBASE = :NRCARDAPIOBASE
                   AND DIACARDBASE.NRORG = :NRORG";
        
        $query = $connection->prepare($sql);
        
        $params = array(
            "NRCARDAPIOBASE" => $nrcardapiobase,
            "NRORG"          => $this->environment->getNrOrgTrab()
        ); 
        
        $query->execute($params);
        
        $result = $query->fetch(\PDO::FETCH_ASSOC);
        
        return $result["DIASEXISTENTES"];
    }
    
}